library(tensorflow)
library(keras)
library(timeSeries)
#0. load the dataset
print(load("./data/UNRATE.RData"))
variable.label="US Unemployment rate, %"

plot(UNRATE,col="steelblue",main=variable.label);grid()

Y=window(UNRATE,start=start(UNRATE),end="2019-12-01")

plot(Y,col="steelblue",main=variable.label);grid()

#1. Choosing the dates of training and testing data
train.start=as.character(start(Y))
train.end="2008-12-01" 
t0=which(as.character(time(Y))==train.end)
test.start=as.character(time(Y))[t0+1];test.start
test.end=as.character(end(Y));test.end

plot(Y,col="steelblue",main=variable.label);grid()
abline(h=0,v=as.POSIXct(test.start),col="red")
text(as.POSIXct(test.start), max(Y)*0.9, test.start,col="blue")

#2. 展開Y的落後結構 AR(p)，以p=2為例
p=2
datasetY=as.timeSeries(embed(Y,p+1),as.Date(time(Y)[-c(1:p)]))
y=datasetY[,1]
colnames(y)="y"
ar=datasetY[,-1]
colnames(ar)=paste0("LY",1:p)

#3. 其餘外生變數X的落後結構, moreData.csv；如果您有解釋變數，於此載入。
temp2=read.csv("moreData.csv")  
moreData=temp2[,-1,drop=FALSE]
rownames(moreData)=as.Date(temp2[,1])
moreData=as.timeSeries(moreData)

indVars=window(moreData[,1:3],start=start(Y),end=end(Y))
datasetX=as.timeSeries(embed(indVars,p+1),as.Date(time(indVars)[-c(1:p)]))
colNAMES=c(outer(paste0(names(indVars),"_L"),0:p,FUN=paste0))
colnames(datasetX)=colNAMES
IDx=grep(colNAMES,pattern="L1") #get X with lag 1

X=datasetX[,IDx]

#4. 定義時間特徵虛擬變數
trend=1:nrow(y)
seasonDummy = data.frame(forecast::seasonaldummy(as.ts(y)))


#5. Input variables
newData= cbind(y, ar, seasonDummy)
tail(newData); head(newData)

#6. 宣告LSTM網路迴圈，越多越精細，但是不要超過觀察值的個數
memoryLoops=200  

### LSTM 估計由以下開始，不需更動
###################################
###=====LSTM modeling begins=====###
###################################

trainData=window(newData, start=start(newData), end=train.end)
testData=window(newData, start=test.start, end=end(newData))

train0 = data.frame(value = as.numeric(trainData[,1]), trainData[,-1])
train = train0[complete.cases(train0), ]

test0 = data.frame(value = as.numeric(testData[,1]), testData[,-1])
test = test0[complete.cases(test0), ]

batch.size = DescTools::GCD(nrow(train),nrow(test))
nrow(train)/batch.size; nrow(test)/batch.size;batch.size

train.new=as.matrix(train) #remove date index
dimnames(train.new)=NULL
test.new=as.matrix(test)  #remove date index
dimnames(test.new)=NULL

SHAPE=5
x.train = array(data = train.new[,-1], dim = c(nrow(train.new), SHAPE, 2))
y.train = array(data = train.new[,1], dim = c(nrow(train.new), 1))

x.test = array(data = test.new[,-1], dim = c(nrow(test.new), SHAPE, 2))
y.test = array(data = test.new[,1], dim = c(nrow(test.new), 1))

model <- keras_model_sequential()

model %>%
  layer_lstm(units = 100,
             input_shape = c(SHAPE, 2),
             batch_size = batch.size,
             return_sequences = TRUE,
             stateful = TRUE) %>%
  layer_dropout(rate = 0.5) %>%
  layer_lstm(units = 50,
             return_sequences = FALSE,
             stateful = TRUE) %>%
  layer_dropout(rate = 0.5) %>%
  layer_dense(units = 1)

model %>%
  compile(loss = 'mae', optimizer = 'adam')

model

system.time(for(i in 1:memoryLoops){
  model %>% fit(x = x.train,
                y = y.train,
                batch_size = batch.size,
                epochs = 1,
                verbose = 0,
                shuffle = FALSE)
  model %>% reset_states()}) 

###################################
###=== Direct Multisteps Forecasts ===###
###################################

# Predict the train data
Pred1_dm <- predict(model,x.train, batch_size = batch.size) %>% .[,1]

# Predict test data: Direct Forecasts 直接預測
Pred2.dm <- predict(model, x.test, batch_size = batch.size) %>% .[,1]

# Calculate Accuracy
LSTM_dm=as.matrix(forecast::accuracy(as.ts(Pred2.dm),x=as.ts(y.test)))
LSTM_dm[,c(4,5,7)]=LSTM_dm[,c(4,5,7)]/100
MAIN.dm=paste0("LSTM_dm: AR(",p,")")
rownames(LSTM_dm)=MAIN.dm;LSTM_dm


#繪圖
sample1=cbind(pred=Pred1_dm,actual=y.train)
sample2=cbind(pred=Pred2.dm,actual=y.test)
year=as.integer(substr(as.character(end(Y)),1,4))
month=as.integer(substr(as.character(end(Y)),6,7))
newDataX=ts(rbind(sample1, sample2),end=c(2019,12),freq = frequency(as.ts(newData)))
newDataX=as.timeSeries(newDataX)
MAIN1=paste0(MAIN.dm," with memory loops=",memoryLoops)

dev.new()
plot(newDataX[,2],ylim=range(newDataX),main=MAIN1,ylab=variable.label,xlab="Direct Forecasts", type="o")
lines(newDataX[,1],ylim=range(newDataX), col="red", type="o")
abline(h=0,v=as.POSIXct(test.start),col="blue");grid()
text(as.POSIXct(rownames(newDataX)[t0-3]), max(Y)*0.9, test.start,col="blue")


######################################
###=== Recursive Multisteps Forecasts ===###
######################################

# Predict test data: Recursive Forecasts 遞迴預測
test0=as.matrix(cbind(test$value, test[,-c(1:2)]))
ar.names=colnames(test0)[grep(colnames(test0),pattern="^ar+")]
rownames(test0)=NULL

ARX=as.matrix(cbind(test[,-c(1:2)]))
ahead=nrow(ARX)
ar.names=colnames(ARX)[grep(colnames(ARX),pattern="^ar+")]
rownames(ARX)=NULL

dynPred=NULL
for (i in 1:ahead) {
  
  x.test0 = array(data = ARX[i,], dim = c(nrow(ARX),SHAPE, 2))
  y0=(model %>% predict(x.test0, batch_size = batch.size) %>% .[,1])[1]
  
  if (i < ahead) {if (length(ar.names)==1) { ARX[i+1,ar.names]=y0
  } else { 
    updates=as.numeric(c(y0,ARX[i,ar.names][1:(p-1)]))
    ARX[i+1,ar.names]=updates }} else {stop}
  
  dynPred=c(dynPred,y0)
  
} 

x.test.new = array(data = ARX, dim = c(nrow(ARX), SHAPE, 2))

Pred2.rm=model %>% predict(x.test.new, batch_size = batch.size) %>% .[,1]

# 計算預測正確度Accuracy
LSTM_rm=as.matrix(forecast::accuracy(as.ts(Pred2.rm),x=as.ts(y.test)))
LSTM_rm[,c(4,5,7)]=LSTM_rm[,c(4,5,7)]/100
MAIN.rm=paste0("LSTM_rm: AR(",p,")")
rownames(LSTM_rm)=MAIN.rm;LSTM_rm

# 繪圖
sample1=cbind(pred=Pred1_dm,actual=y.train)
sample2=cbind(pred=Pred2.rm,actual=y.test)
year=as.integer(substr(as.character(end(Y)),1,4))
month=as.integer(substr(as.character(end(Y)),6,7))
newDataX=ts(rbind(sample1, sample2),end=c(year,month),freq = frequency(as.ts(Y)))
newDataX=as.timeSeries(newDataX)
MAIN2=paste0(MAIN.rm," with memory loops=", memoryLoops)

dev.new()
plot(newDataX[,2],ylim=range(newDataX),main=MAIN2,ylab=variable.label,xlab="Recursive Forecasts", type="o")
lines(newDataX[,1],ylim=range(newDataX), col="red", type="o")
abline(h=0,v=as.POSIXct(test.start),col="blue");grid()
text(as.POSIXct(rownames(newDataX)[t0-10]), max(Y)*0.9, test.start,col="blue")

LSTM=round(rbind(LSTM_dm, LSTM_rm),4);LSTM
#save(LSTM,file="./output/data/LSTM.RData")
