#Case 1. Fed
url="https://fred.stlouisfed.org/"
url1="https://fred.stlouisfed.org/graph/fredgraph.csv?&id=USACPIALLMINMEI&fq=Monthly&revision_date=2022-05-01&nd=1947-01-01"
url2="https://fred.stlouisfed.org/graph/fredgraph.csv?id=UNRATE&fq=Monthly&revision_date=2021-08-02&nd=1948-01-01"
url3="https://fred.stlouisfed.org/graph/fredgraph.csv?id=GDP&fq=Quarterly&&revision_date=2021-08-02&nd=1947-01-01"
url4="https://fred.stlouisfed.org/graph/fredgraph.csv?&id=GDPC1&fq=Quarterly&revision_date=2021-08-02&nd=1947-01-01"
library(timeSeries)
dat=read.csv(url1)
tail(dat)
dateID=as.Date(dat[,1])
y0=dat[,2,drop=FALSE]
us_cpi=window(timeSeries(y0,dateID),start="1960-01-01",end="2022-04-01")
tail(us_cpi)
y=diff(log(us_cpi),12)*100

dev.new();par(mfrow=c(2,1))
plot(y,col="steelblue",xlab="",ylab="", main="US Inflation Rate,%");grid();abline(h=0,col="red")
axis(4,at=round(as.numeric(y[end(y)]),2),col="blue");
plot(us_cpi,col="steelblue",ylab="",main="US CPI, 2015=100");grid()
par(mfrow=c(1,1))

#Case 2. TWSE 5-second
twse_5sec="https://www.twse.com.tw/exchangeReport/MI_5MINS_INDEX?response=csv&date=20210806"
dat0=read.csv(twse_5sec,skip=2)
tail(dat0,15)

dat=dat0[1:3241,-ncol(dat0)]
dateTime=sub("=","",dat[,1])

#Case 3. BIS effective exchange rates
library(openxlsx)
eer_broad="https://www.bis.org/statistics/eer/broad.xlsx"
temp1 = read.xlsx(eer_broad, sheet=1, colNames=T, detectDates = T, startRow = 4)
head(temp1)
countryNames=colnames(temp1)[-1]
temp2 = read.xlsx(eer_broad, sheet=1, colNames=T, detectDates = T, startRow = 5)
tail(temp2)
dateID=as.character(temp2[,1])
eer_data=temp2[,-1]



#Case 4. quantmod

library(quantmod)
indexnames=c("^DJI","^GSPC","^IXIC","^RUT","^VIX","^SOX")
filenames=c("DJI","GSPC","IXIC","RUT","VIX","SOX")
getSymbols(indexnames,from="1926-01-01", adjust=TRUE)
save("DJI","GSPC","IXIC","RUT","VIX","SOX",file="./data/index.RData")
print(load("./data/index.RData"))

SPDR=c("XLB","XLE","XLF","XLP","XLI","XLU","XLV","XLK","XLY","RWR","SPY")
getSymbols(SPDR,from="1990-01-01", to=as.Date(Sys.time()), adjust=TRUE)
save("XLB","XLE","XLF","XLP","XLI","XLU","XLV","XLK","XLY","RWR","SPY", file="./data/SPDR.RData")
library(xts)
print(load("./data/SPDR.RData"));tail(RWR)

iShares=c("EWJ","EWG","EWU","EWC","EWY","EWA","EWH","EWS","IYZ","EZU","IYR","EWT","EWZ","EFA","IGE","EPP","LQD","SHY","IEF","TLT")
getSymbols(iShares,from="1990-01-01", to=as.Date(Sys.time()),adjust=TRUE)
save("EWJ","EWG","EWU","EWC","EWY","EWA","EWH","EWS","IYZ","EZU","IYR","EWT","EWZ","EFA","IGE","EPP","LQD","SHY","IEF","TLT", file="./data/iShares.RData")
print(load("./data/iShares.RData"));tail(EWS)


#Case 5. Fama-French 

###==== Download Factors from French's Web 
url="http://mba.tuck.dartmouth.edu/pages/faculty/ken.french/ftp/"
filename=c(
  "F-F_Research_Data_Factors",
  "F-F_Research_Data_Factors_weekly",
  "F-F_Research_Data_Factors_daily",
  "F-F_Research_Data_5_Factors_2x3",
  "F-F_Research_Data_5_Factors_daily_2x3",
  "F-F_Momentum_Factor",
  "Portfolios_Formed_on_ME",
  "100_Portfolios_ME_INV_10x10",
  "49_Industry_Portfolios",
  "30_Industry_Portfolios")
k=10
fileZIP=paste0(filename[k],"_CSV.zip")
Link=paste0(url,fileZIP)
temp = tempfile(tmp=getwd(),fileext = ".zip")
download.file(Link,temp)
fileCSV=paste0(filename[k],".CSV")
ff.factor0 = read.csv(unz(temp, fileCSV),skip=0,header=FALSE, blank.lines.skip = FALSE) 
head(ff.factor0,15)
ff.factor0 = read.csv(unz(temp, fileCSV),skip=10,header=FALSE, blank.lines.skip = FALSE) 
head(ff.factor0)

table.names=c("  Average Value Weighted Returns -- Monthly", 
              "  Average Equal Weighted Returns -- Monthly",
              "  Average Value Weighted Returns -- Annual",
              "  Average Equal Weighted Returns -- Annual",
              "  Average Firm Size",
              "  Sum of BE / Sum of ME",
              "  Value-Weighted Average of BE/ME")

colNAMES=as.character(ff.factor0[2,][-1])
table.names.where=which(ff.factor0[,1] %in% table.names)

VWP.mon=ff.factor0[(table.names.where[1]+2):(table.names.where[2]-3),]
head(VWP.mon,3)
tail(VWP.mon,3)
colnames(VWP.mon)=c("dates",colNAMES)
head(VWP.mon,3)
tail(VWP.mon,3)

EWP.mon=ff.factor0[(table.names.where[2]+2):(table.names.where[3]-3),]
colnames(EWP.mon)=c("dates",colNAMES)
head(EWP.mon,3)
tail(EWP.mon,3)

VWP.ann=ff.factor0[(table.names.where[3]+2):(table.names.where[4]-3),]
colnames(VWP.ann)=c("dates",colNAMES)
head(VWP.ann,3)
tail(VWP.ann,3)

EWP.ann=ff.factor0[(table.names.where[4]+2):(table.names.where[5]-3),]
colnames(EWP.ann)=c("dates",colNAMES)
head(EWP.ann,3)
tail(EWP.ann,3)

firm.size.mon=ff.factor0[(table.names.where[5]+2):(table.names.where[6]-3),]
colnames(firm.size.mon)=c("dates",colNAMES)
head(firm.size.mon,3)
tail(firm.size.mon,3)

sumBE_sumME=ff.factor0[(table.names.where[6]+2):(table.names.where[7]-3),]
colnames(sumBE_sumME)=c("dates",colNAMES)
head(sumBE_sumME,3)
tail(sumBE_sumME,3)


VW_avgBE_ME=ff.factor0[(table.names.where[7]+2):(length(ff.factor0[,1])-1),]
colnames(VW_avgBE_ME)=c("dates",colNAMES)
head(VW_avgBE_ME,3)
tail(VW_avgBE_ME,3)


#NOTES
grep(ff.factor0[,1],pattern=" Average")
grep(ff.factor0[,1],pattern="  Sum")


#Case 6 Download Stock AI

source("./src/utilities.R")

downloadStockAI(var.name="TWECO")


