# Simulate an ARIMA process
ar1.sim0=arima.sim(n = 200, list(ar = c(0.5, -0.25)))

ar1.sim1=ts(ar1.sim0, end=c(2021,8),freq=12)

ar1.sim2=timeSeries::as.timeSeries(ar1.sim1)


dev.new();par(mfrow=c(2,1))
plot(ar1.sim1,col="steelblue",ylab="AR(1)", xlab="(A) 使用 plot() 繪製 ts() 物件")
grid()

plot(ar1.sim2,col="steelblue",ylab="AR(1)",xlab="(B) 使用 plot() 繪製 timeSeries() 物件")
grid()
rug(as.vector(ar1.sim2), ticksize = 0.01, side = 2, quiet = TRUE)
(mfrow=c(1,1))

arima.sim(n = 300, list(ar = c(0.5), ma = c(-0.25)))
arima.sim(n = 300, list(ar = c(0.5), ma = c(-0.25)),sd = sqrt(0.16))

arima.sim(list(order = c(0,1,0)), n = 600)
arima.sim(list(order = c(1,1,1), ar = c(0.125),ma=c(-0.225)), n = 600)

dev.new();plot(arima.sim(list(order = c(1,1,1), ar = c(0.125),ma=c(-0.225)), n = 600),ylab="",main="arima.sim(list(order = c(1,1,1), ar = c(0.125),ma=c(-0.225)), n = 600)")

# mildly long-tailed
arima.sim(n = 3000, list(ar = c(0.88, -0.48), ma = c(-0.22, 0.28)),rand.gen = function(n, ...) sqrt(0.16) * rt(n, df = 5))

TEXT="arima.sim(n = 600, list(ar = c(0.88, -0.48), ma = c(-0.22, 0.28)),rand.gen = function(n, ...) sqrt(0.16) * rt(n, df = 5))"

dev.new();plot(arima.sim(n = 3000, list(ar = c(0.88, -0.48), ma = c(-0.22, 0.28)),
                          rand.gen = function(n, ...) sqrt(0.16) * rt(n, df = 5)), ylab="",main=TEXT)


library(forecast)
library(timeSeries)

#Case 1. ARIMA
print(load("./data/tw_urate.RData"))
y=tw_urate
auto.arima(y,ic="aic")
E_y=as.timeSeries(fitted(auto.arima(y)))

dev.new();plot(y,col="steelblue",xlab="Unemployment rate of Taiwan, %",main="auto.arima(): ARIMA(0,1,4)(2,0,0)[12]",ylab="");grid()
lines(E_y,col="red",lty=2)
rug(as.vector(y), ticksize = 0.01, side = 2, quiet = TRUE)
abline(v=as.POSIXct("2019-11-01"),col="blue",lty=3)


y_training=as.ts(window(y,start="1978-01-01",end="2019-11-01"))
y_testing=as.ts(window(y,start="2019-12-01",end="2021-07-01"))

dev.new();plot(y,col="steelblue",xlab="Unemployment rate of Taiwan, %",main="auto.arima(): ARIMA(0,1,4)(2,0,0)[12]",ylab="");grid()
rug(as.vector(y), ticksize = 0.01, side = 2, quiet = TRUE)
abline(v=as.POSIXct("2019-11-01"),col="red",lty=3)


y.auto=auto.arima(y_training,ic="aic")
arimaOrder=arimaorder(y.auto) 
arimaOrder
common=arimaOrder[c("p","d","q")]
seasonal=arimaOrder[c("P","D","Q")]

y.arma=Arima(y_training,order=common,seasonal)
fcst1 = Arima(model=y.arma, y_testing)$fitted
accuracy(fcst1,x=y_testing)


fcst2 = forecast(y.arma, h=20)$mean
accuracy(fcst2,x=y_testing)


dev.new();
par(mfrow=c(2,1))
plot(y,col="steelblue",xlab="(A) Static Forecasts",main="Unemployment rate of Taiwan, %",ylab="");grid()
rug(as.vector(y), ticksize = 0.01, side = 2, quiet = TRUE)
abline(v=as.POSIXct("2019-11-01"),col="red",lty=3)
lines(as.timeSeries(fcst1),col="red")


plot(y,col="steelblue",xlab="(B) Dynamic Forecasts",main="",ylab="");grid()
rug(as.vector(y), ticksize = 0.01, side = 2, quiet = TRUE)
abline(v=as.POSIXct("2019-11-01"),col="red",lty=3)
lines(timeSeries(fcst2,time(as.timeSeries(fcst1))),col="red")

par(mfrow=c(1,1))




#source("./src/rollingWindows.R")
timeframe=iForecast::rollingWindows(y,training ="120m",testing ="18m")
FROM=timeframe$from
TO=timeframe$to


#Case 2. SETAR/LSTAR. May not work for small out-of-sample
# SETAR
library(tsDyn)

y_training=as.ts(window(y,start="2010-01-02",end="2021-06-30"))
y_testing=as.ts(window(y,start="2021-07-01",end="2021-09-03"))

m=4
out.setar=setar(y_training,m,mL=3, mH=2)
coefs=out.setar$coef
th=coefs["th"]
mL=out.setar$model.specific$mL
mH=out.setar$model.specific$mH
thVar=y.nL1$model.specific$thVar
mTh=out.setar$model.specific$mTh
out.setar.fcst1=setar(y_testing, m,th=th,mL=mL,mH=mH,mTh=mTh)$fitted
out.setar.fcst1=ts(out.setar.fcst1,end=end(y_testing),freq=frequency(y_testing))
#options(scipen = 999)
round(forecast::accuracy(out.setar.fcst1,x=y_testing),6)

out.setar.fcst2=predict(out.setar,n.ahead = length(y_testing))
out.setar.fcst2=ts(out.setar.fcst2,end=end(y_testing),freq=frequency(y_testing))
forecast::accuracy(out.setar.fcst2,x=y_testing)
#dev.new();plot(out.setar)



# LSTAR. May not work for small out-of-sample
out.lstar=lstar(y_training,m,mL=3, mH=2)
coefs=out.lstar$coef
gamma=coefs["gamma"]
th=coefs["th"]
mL=out.lstar$model.specific$mL
mH=out.lstar$model.specific$mH
thVar=out.lstar$model.specific$thVar
mTh=out.lstar$model.specific$mTh
out.lstar.fcst1=lstar(y_testing, m,gamma=gamma,th=th,mL=mL,mH=mH,mTh=mTh)$fitted
out.lstar.fcst1=ts(out.lstar.fcst1,end=end(y_testing),freq=frequency(y_testing))
round(forecast::accuracy(out.lstar.fcst1,x=y_testing),6)

out.lstar.fcst2=predict(out.lstar,n.ahead = length(y_testing))
out.lstar.fcst2=ts(out.lstar.fcst2,end=end(y_testing),freq=frequency(y_testing))
forecast::accuracy(out.lstar.fcst2,x=y_testing)


#BATS model
library(forecast)
library(timeSeries)
print(load("./data/us_cpi.RData"))
y=na.omit(diff(log(us_cpi),12))*100
y_training=as.ts(window(y,start="1948-01-01",end="2019-12-31"))
y_testing=as.ts(window(y,start="2020-01-01",end="2021-09-03"))

out.bats = bats(y_training, use.box.cox = TRUE, use.trend = TRUE, use.damped.trend = TRUE)
out.bats.fcst1 = bats(model=out.bats,y_testing)$fitted
accuracy(out.bats.fcst1,x=y_testing)

out.bats.fcst2 =forecast(out.bats,h=length(y_testing))$mean
accuracy(out.bats.fcst2,x=y_testing)

#Bagged model 1. ETS
bagged.ets = baggedModel(y_training,fn = ets)
M1=bagged.ets$models[[which.min(sapply(bagged.ets$models, BIC))]]
ets.fcst1=baggedETS(model=M1,y_testing)$fitted
accuracy(ets.fcst1,x=y_testing)

ets.fcst2 =forecast(M1,h=length(y_testing))$mean
accuracy(ets.fcst2, x=y_testing)

ets.fcst3=forecast(out.bagged.ets,h=length(y_testing))$mean
accuracy(ets.fcst3,x=y_testing)

#Bagged model 2. auto.arima
bagged.arima = baggedModel(y_training,fn = auto.arima)
M2=bagged.arima$models[[which.min(sapply(bagged.arima$models,BIC))]]

arima.fcst1 = Arima(model=M2,y_testing)$fitted
accuracy(arima.fcst1,x=y_testing)

arima.fcst2 =forecast(M2,h=length(y_testing))$mean
accuracy(arima.fcst2,x=y_testing)

arima.fcst3 =forecast(bagged.arima,h=length(y_testing))$mean
accuracy(arima.fcst3,x=y_testing)


#Averaging static forecasts
prediction.ets=lapply(bagged.ets$models,function(x) baggedETS(model=x,y_testing)$fitted)
prediction.ets=as.data.frame(prediction.ets)
colnames(prediction.ets)=paste0("P",1:100)
yf.ets=ts(apply(prediction.ets,1,median),start=start(y_testing),frequency = frequency(y_testing))
accuracy(yf.ets,x=y_testing)


prediction.arima=lapply(bagged.arima$models,function(x) Arima(model=x,y_testing)$fitted)
prediction.arima=as.data.frame(prediction.arima)
colnames(prediction.arima)=paste0("P",1:100)
yf.arima=ts(apply(prediction.arima,1,median),start=start(y_testing),frequency = frequency(y_testing))
accuracy(yf.arima,x=y_testing)


#GAMS model
p=4
newData = embed(y,p+1)
dat = ts(newData,end=end(as.ts(y)),freq=frequency(y))
dat=as.timeSeries(dat)
data_training=window(dat,start=start(dat),end="2019-12-31")
data_testing=window(dat,start="2020-01-01",end="2021-09-03")
colnames(data_training)=colnames(data_testing)=c("dep",paste0("ar",1:(ncol(dat)-1)))

eq = as.formula(paste("dep ~", paste(colnames(data_testing)[-1], collapse = "+")))
gam1 = mgcv::gam(eq, data = data_testing)
newdata=as.data.frame(data_testing)
gam1.fcst=as.ts(as.timeSeries(as.data.frame(predict(gam1,newdata))))
accuracy(gam1.fcst,x=as.ts(data_testing[,"dep"]))

gam2=tsDyn::aar(as.ts(window(y,start=start(y),end="2019-12-31")),m=p)
gam2.fcst=predict(gam2, n.ahead = nrow(data_testing))
accuracy(gam2.fcst, x=as.ts(data_testing[,"dep"]))


