#library(iForecast)
library(timeSeries)
library(forecast)

print(load("./data/tw_IPI.RData"))
Y=na.omit(diff(log(IPI),12)*100)
head(Y)
tail(Y,10)
train.end="2018-08-01"
test.start="2018-09-01"
t0=which(as.character(time(Y))==train.end)
dev.new();plot(Y,col="steelblue",main="臺灣月工業生產指數",ylab="%",xlab="時間",lwd=2);grid()
abline(h=0,v=as.POSIXct(test.start),col="red", lty=2);grid()
text(as.POSIXct(rownames(Y)[t0-10]), max(Y)*0.9, test.start,col="blue")

tainingData=window(Y,start=start(Y),end=train.end)
testData=window(Y,start=test.start,end=end(Y))
nrow(tainingData)
nrow(testData)

#####==== Estimation and Validation by Static Prediction =====#####

timeframe=iForecast::rollingWindows(rev(tainingData),training="360m",testing="12m")
FROM=timeframe$from
TO=timeframe$to

ya=as.ts(testData)
  
ahead=nrow(testData)

FCST=list()
Model1=Model2=Model3=Model4=Model5=Model6=Model7=Model8=Model9=Model10=Model11=Model12=list()
ACC1=ACC2=ACC3=ACC4=ACC5=ACC6=ACC7=ACC8=ACC9=ACC10=ACC11=ACC12=NULL
start0=Sys.time();for (t0 in 1:length(TO)) { #  t0=1
  
  y=as.ts(window(tainingData,start=TO[t0],end=FROM[1]))
  

#=================================#
# Forecasting Econometric Models  #
#=================================#

#Step 1. Using auto.arima() to select the optimal orders.
y.auto=auto.arima(y,ic="bic")
arimaOrder=arimaorder(y.auto) 
if(is.na(arimaOrder[c("P","D","Q")][1])) {seasonal=c(0, 0, 0)
} else {seasonal=arimaOrder[c("P","D","Q")]}

#Step 2. We  prepare legend m1 for labeling ARIMAs results.
order1=paste0(arimaOrder["p"],",",arimaOrder["d"],",",arimaOrder["q"])
if(anyNA(arimaOrder[c("P","D","Q")])) {order2=c("0,0,0")
} else {order2=paste0(arimaOrder["P"],",",arimaOrder["D"],",",arimaOrder["Q"])}

model.name=paste0("ARIMA(",order1,")(",order2,")")

# Model 1. Pure ARIMA
y.arma1=Arima(y,order=arimaOrder[c("p","d","q")],seasonal)
fcst1 <- forecast(y.arma1, h=ahead)$mean
acc1=accuracy(fcst1,x=ya)
rownames(acc1)=model.name
Model1[[t0]]=y.arma1

# Model 2. Pure ARIMA with seasonal dummies
y.arma2=Arima(y,order=arimaOrder[c("p","d","q")],seasonal,xreg=seasonaldummy(y))
fcst2 <- forecast(y.arma2, h=ahead,xreg=seasonaldummy(ya))$mean
acc2=accuracy(fcst2,x=ya)
rownames(acc2)=paste0(model.name,"+sDummy")
Model2[[t0]]=y.arma2

# Model 3. Pure ARIMA with seasonal dummies and trend
y.arma3=Arima(y,order=arimaOrder[c("p","d","q")],seasonal,xreg=cbind(seasonaldummy(y),seq(y)))
fcst3 <- forecast(y.arma3, h=ahead,xreg=cbind(seasonaldummy(ya),seq(ya)))$mean
acc3=accuracy(fcst3,x=ya)
rownames(acc3)=paste0(model.name,"+trend+sDummy")
Model3[[t0]]=y.arma3

# Model 4. Pure ARIMA with Fourier cycles
y.fourier <- Arima(y,order=arimaOrder[c("p","d","q")],seasonal, xreg=cbind(fourier(y,K=2)))
fcst4 <- forecast(y.fourier, xreg=fourier(y, K=2, h=ahead))$mean
acc4=accuracy(fcst4,x=ya)
rownames(acc4)=paste0(model.name,"+Fourier")
Model4[[t0]]=y.fourier

# Model 5 Pure ARIMA with Fourier cycles and trend
XREG=cbind(fourier(y,K=2),trend=seq(y))
y.fourier <- Arima(y,order=arimaOrder[c("p","d","q")],seasonal, xreg=XREG)
fcst5 <- forecast(y.fourier, xreg=cbind(fourier(ya, K=2),trend=seq(ya)))$mean
acc5=accuracy(fcst5,x=ya)
rownames(acc5)=paste0(model.name,"+Fourier+trend")
Model5[[t0]]=y.fourier                      


# Model 6 Bagged method: Data fitted by ets, exogenous variables not allowed
y.bagged.ets <- baggedModel(y,fn = ets)
fcst6 <- forecast(y.bagged.ets,h=ahead, use.initial.values=TRUE)$mean
acc6=accuracy(fcst6,x=ya)
rownames(acc6)="bagged.ETS"
Model6[[t0]]=y.bagged.ets

# Model 7 Bagged method: Data fitted by auto.arima, exogenous variables not allowed
y.bagged.auto.arima <- baggedModel(y,fn = auto.arima)
fcst7=forecast(y.bagged.auto.arima,h=ahead, use.initial.values=TRUE)$mean
acc7=accuracy(fcst7,x=ya)
rownames(acc7)="bagged.auto.arima"
Model7[[t0]]=y.bagged.auto.arima


# Model 8 Neural Net AR, exogenous variables not allowed
y.nnetar <- nnetar(y, decay=0.85, maxit=200, size=4)
fcst8 <-forecast(y.nnetar,h=ahead)$mean
acc8=accuracy(fcst8,x=ya)
rownames(acc8)="NNETAR"
Model8[[t0]]=y.nnetar


# Model 9 BATS model, exogenous variables not allowed
y.bats <- bats(y,use.box.cox = TRUE, use.trend = TRUE, use.damped.trend = TRUE)
fcst9 <-forecast(y.bats,h=ahead)$mean
acc9=accuracy(fcst9,x=ya)
rownames(acc9)="BATS"
Model9[[t0]]=y.bats


# Model 10. ARFIMA, exogenous variables not allowed
y.arfima=arfima(y, drange = c(0, 0.8), estim = c("mle", "ls")[1], model = NULL,
                lambda = NULL, biasadj = T)
fcst10 <- forecast(y.arfima, h=ahead)$mean
acc10=accuracy(fcst10,x=ya)
rownames(acc10)="ARFiMA"
Model10[[t0]]=y.arfima


# Model 11. Non-linear: SETAR
y.nl1=setar(y,m=4)
fcst11=predict(y.nl1,n.ahead = ahead)
acc11=accuracy(fcst11,x=ya)
rownames(acc11)="SETAR"
Model11[[t0]]=y.nl1

# Model 12. Non-linear: GAMS
y.nl2=aar(y, m=4)
fcst12=predict(y.nl2,n.ahead = ahead)
acc12=accuracy(fcst12,x=ya)
rownames(acc12)="GAM"
Model12[[t0]]=y.nl2

# Model 13. Non-linear: LSTAR
#library(tsDyn)
#m=4
#y.nl3=lstar(y,m)
#fcst13=predict(y.nl3,n.ahead = ahead)
#acc13=accuracy(fcst13,x=ya)
#rownames(acc13)="LSTAR"
#Model13[[t0]]=y.nl3


predicted=cbind(ya,fcst1,fcst2,fcst3,fcst4,fcst5,fcst6,fcst7,fcst8,fcst9,fcst10,fcst11,fcst12)
colnames(predicted)=c("ya",rownames(acc1),rownames(acc2),rownames(acc3),rownames(acc4),rownames(acc5),rownames(acc6),
                      rownames(acc7),rownames(acc8),rownames(acc9),rownames(acc10),rownames(acc11),rownames(acc12))
FCST[[t0]]=predicted


ACC1=rbind(ACC1,acc1)
ACC2=rbind(ACC2,acc2)
ACC3=rbind(ACC3,acc3)
ACC4=rbind(ACC4,acc4)
ACC5=rbind(ACC5,acc5)
ACC6=rbind(ACC6,acc6)
ACC7=rbind(ACC7,acc7)
ACC8=rbind(ACC8,acc8)
ACC9=rbind(ACC9,acc9)
ACC10=rbind(ACC10,acc10)
ACC11=rbind(ACC11,acc11)
ACC12=rbind(ACC12,acc12)


print(paste(t0,length(TO),sep="/"))
};end0=Sys.time()

end0-start0

ACC1[,c(4,5,7)]=ACC1[,c(4,5,7)]/100
ACC2[,c(4,5,7)]=ACC2[,c(4,5,7)]/100
ACC3[,c(4,5,7)]=ACC3[,c(4,5,7)]/100
ACC4[,c(4,5,7)]=ACC4[,c(4,5,7)]/100
ACC5[,c(4,5,7)]=ACC5[,c(4,5,7)]/100
ACC6[,c(4,5,7)]=ACC6[,c(4,5,7)]/100
ACC7[,c(4,5,7)]=ACC7[,c(4,5,7)]/100
ACC8[,c(4,5,7)]=ACC8[,c(4,5,7)]/100
ACC9[,c(4,5,7)]=ACC9[,c(4,5,7)]/100
ACC10[,c(4,5,7)]=ACC10[,c(4,5,7)]/100
ACC11[,c(4,5,7)]=ACC11[,c(4,5,7)]/100
ACC12[,c(4,5,7)]=ACC12[,c(4,5,7)]/100


ACC=list();for (i in 1:12){
tmp=paste0("ACC",i)
ACC[[i]]=round(eval(parse(text=tmp)),4)
}

save(ACC,FCST,Model1,Model2,Model3,Model4,Model5,Model6,Model7,Model8,Model9,Model10,
     Model11,Model12,testData, file="./output/data/arima_recursive.RData")



#Generate Forecasts of test data
print(load("./output/data/arima_recursive.RData"))
library(forecast)


which.min(ACC[[8]][,"RMSE"])
t.id=12
Model8[[t.id]]
MODEL.used=Model8[[t.id]]
fcst <-forecast(MODEL.used,h=nrow(testData))$mean
acc=as.matrix(accuracy(as.ts(fcst),x=as.ts(testData)))
acc[,c(4,5,7)]=acc[,c(4,5,7)]/100
acc=round(acc,4)
acc


  MODEL.used=Model8[[13]]
  fcst <-forecast(MODEL.used,h=nrow(testData))$mean
  accx=as.matrix(accuracy(as.ts(fcst),x=as.ts(testData)))
  accx[,c(4,5,7)]=accx[,c(4,5,7)]/100
  accx=round(accx,4)
  accx


FCSTx=NULL
for (k in 1:12) {
  MODEL.used=Model8[[k]]$models[[length(Model8[[k]]$models)]]
  fcst <- Arima(model=MODEL.used,testData)$fitted
  FCSTx=cbind(FCSTx,fcst)
}
colnames(FCSTx)=paste0("Model",1:13)
fcstAvg=as.matrix(apply(FCSTx,1,mean))
rownames(fcstAvg)=as.character(time(testData))
accuracy(as.ts(as.timeSeries(fcstAvg)),as.ts(testData))


sort(c(ACCX,4.766202))
