#library(iForecast)
library(timeSeries)
library(forecast)

print(load("./data/tw_IPI.RData"))
Y=na.omit(diff(log(IPI),12)*100)
head(Y)
tail(Y,10)
train.end="2018-08-01"
test.start="2018-09-01"
t0=which(as.character(time(Y))==train.end)
dev.new();plot(Y,col="steelblue",main="臺灣月工業生產指數",ylab="%",xlab="時間",lwd=2);grid()
abline(h=0,v=as.POSIXct(test.start),col="red", lty=2);grid()
text(as.POSIXct(rownames(Y)[t0-10]), max(Y)*0.9, test.start,col="blue")

tainingData=window(Y,start=start(Y),end=train.end)
testData=window(Y,start=test.start,end=end(Y))
nrow(tainingData)
nrow(testData)

#####==== Estimation and Validation by Static Prediction =====#####

timeframe=iForecast::rollingWindows(tainingData,training="240m",testing="36m")
FROM=timeframe$from
TO=timeframe$to

FCST=list()
Model1=Model2=Model3=Model4=Model5=Model6=Model7=Model8=Model9=Model10=list()
ACC1=ACC2=ACC3=ACC4=ACC5=ACC6=ACC7=ACC8=ACC9=ACC10=NULL
start0=Sys.time();for (t0 in 1:length(TO)) {  #t0=1
  
  y=window(tainingData,start=FROM[1],end=TO[t0])
  
  if(t0==length(TO)) {
    ya=window(tainingData,start=TO[t0],end=end(tainingData))[-1,]
  } else {ya=window(tainingData,start=TO[t0],end=TO[t0+1])[-1,]}
  
  
  if (nrow(y)<60) {next}
  
  y=as.ts(y)
  ya=as.ts(ya)
  #=====================================#
  # Econometric Time Series forecasting #
  #=====================================#
  
  #Step 1. Using auto.arima() to select the optimal orders.
  y.auto=auto.arima(y)
  arimaOrder=arimaorder(y.auto) 
  #summary(y.auto)
  if(is.na(arimaOrder[c("P","D","Q")][1])) {seasonal=c(0, 0, 0)
  } else {seasonal=arimaOrder[c("P","D","Q")]}
  
  #Step 2. We  prepare legend m1 for labeling ARIMAs results.
  order1=paste0(arimaOrder["p"],",",arimaOrder["d"],",",arimaOrder["q"])
  if(anyNA(arimaOrder[c("P","D","Q")])) {order2=c("0,0,0")
  } else {order2=paste0(arimaOrder["P"],",",arimaOrder["D"],",",arimaOrder["Q"])}
  
  model.name1=paste0("ARIMA(",order1,")(",order2,")")
  model.name2=paste0("ARIMA(",order1,")")
  #====================================#
  # Modeling and Estimation begin here #
  #====================================#
  
  # Model 1. SARIMA
  y.arma1=Arima(y,order=arimaOrder[c("p","d","q")],seasonal)
  fcst1 <- Arima(ya,model=y.arma1)$fitted
  acc1=accuracy(fcst1,x=ya)["Test set",]
  acc1=t(as.matrix(acc1))
  rownames(acc1)=model.name1
  Model1[[t0]]=y.arma1
  
  # Model 2. SARIMA with seasonal dummies
  y.arma2=Arima(y,order=arimaOrder[c("p","d","q")],seasonal, xreg=seasonaldummy(y))
  fcst2 <-Arima(ya,model=y.arma2,xreg=seasonaldummy(ya))$fitted
  acc2=accuracy(fcst2,x=ya)["Test set",]
  acc2=t(as.matrix(acc2))
  rownames(acc2)=paste0(model.name1,"+trend")
  Model2[[t0]]=y.arma2
  
  # Model 3. SARIMA with Fourier cycles
  y.fourier <- Arima(y,order=arimaOrder[c("p","d","q")],seasonal, xreg=cbind(fourier(y,K=4)))
  fcst3 <- Arima(model=y.fourier, ya, xreg=fourier(ya, K=4))$fitted
  acc3=accuracy(fcst3,x=ya)["Test set",]
  acc3=t(as.matrix(acc3))
  rownames(acc3)=paste0(model.name1,"+Fourier")
  Model3[[t0]]=y.fourier
  
  # Model 4. Pure ARIMA with trend
  y.arma4=Arima(y,order=arimaOrder[c("p","d","q")],xreg=seq(y))
  fcst4 <- Arima(model=y.arma4, ya,xreg=seq(ya))$fitted
  acc4=accuracy(fcst4,x=ya)
  acc4=as.matrix(acc4)
  rownames(acc4)=paste0(model.name2,"+trend")
  Model4[[t0]]=y.arma4
  
  # Model 5. Pure ARIMA with seasonal dummies
  y.arma5=Arima(y,order=arimaOrder[c("p","d","q")],xreg=cbind(seasonaldummy(y)))
  fcst5 <- Arima(model=y.arma5, ya,xreg=cbind(seasonaldummy(ya)))$fitted
  acc5=accuracy(fcst5,x=ya)
  acc5=as.matrix(acc5)
  rownames(acc5)=paste0(model.name2,"+sDummy")
  Model5[[t0]]=y.arma5
  
  
  # Model 6. Pure ARIMA with seasonal dummies and trend
  y.arma6=Arima(y,order=arimaOrder[c("p","d","q")],xreg=cbind(seasonaldummy(y),seq(y)))
  fcst6 <- Arima(model=y.arma6, ya,xreg=cbind(seasonaldummy(ya),seq(ya)))$fitted
  acc6=accuracy(fcst6,x=ya)
  acc6=as.matrix(acc6)
  rownames(acc6)=paste0(model.name2,"+trend+sDummy")
  Model6[[t0]]=y.arma6
  
  # Model 7. Bagged method: Data fitted by ets, exogenous variables not allowed
  y.bagged.ets <- baggedModel(y,fn = ets)
  MODEL.ets=y.bagged.ets$models[[length(y.bagged.ets$models)]]
  fcst7 <- baggedETS(model=MODEL.ets,ya, use.initial.values=TRUE)$fitted
  acc7=accuracy(fcst7,x=ya)["Test set",]
  acc7=t(as.matrix(acc7))
  rownames(acc7)="bagged.ETS"
  Model7[[t0]]=y.bagged.ets
  
  # Model 8. Bagged method: Data fitted by auto.arima, exogenous variables not allowed
  y.bagged.auto.arima <- baggedModel(y,fn = auto.arima)
  MODEL.autoARIMA=y.bagged.auto.arima$models[[length(y.bagged.auto.arima$models)]]
  fcst8 <- Arima(model=MODEL.autoARIMA,ya, use.initial.values=TRUE)$fitted
  acc8=accuracy(fcst8,x=ya)["Test set",]
  acc8=t(as.matrix(acc8))
  rownames(acc8)="bagged.auto.arima"
  Model8[[t0]]=y.bagged.auto.arima
  
  # Model 9 BATS, exogenous variables not allowed
  y.bats <- bats(y,use.box.cox = TRUE, use.trend = TRUE, use.damped.trend = TRUE)
  fcst9 <-bats(model=y.bats,ya)$fitted
  acc9=accuracy(fcst9,x=ya)["Test set",]
  acc9=t(as.matrix(acc9))
  rownames(acc9)="BATS"
  Model9[[t0]]=y.bats
  
  # Model 10. ARFIMA, exogenous variables not allowed
  y.arfima=arfima(y, drange = c(0, 0.8), estim = c("mle", "ls")[1], model = NULL,
                  lambda = NULL, biasadj = T)
  fcst10 <- arfima(model=y.arfima, ya)$fitted
  acc10=accuracy(fcst10,x=ya)["Test set",]
  acc10=t(as.matrix(acc10))
  rownames(acc10)="ARFiMA"
  Model10[[t0]]=y.arfima
  
  ACC1=rbind(ACC1,acc1)
  ACC2=rbind(ACC2,acc2)
  ACC3=rbind(ACC3,acc3)
  ACC4=rbind(ACC4,acc4)
  ACC5=rbind(ACC5,acc5)
  ACC6=rbind(ACC6,acc6)
  ACC7=rbind(ACC7,acc7)
  ACC8=rbind(ACC8,acc8)
  ACC9=rbind(ACC9,acc9)
  ACC10=rbind(ACC10,acc10)
  
  predicted=cbind(ya,fcst1,fcst2,fcst3,fcst4,fcst5,fcst6,fcst7,fcst8,fcst9,fcst10)
  colnames(predicted)=c("ya",rownames(acc1),rownames(acc2),
                        rownames(acc3),rownames(acc4),rownames(acc5),
                        rownames(acc6),rownames(acc7),rownames(acc8),
                        rownames(acc9),rownames(acc10))
  FCST[[t0]]=predicted
  
  print(paste(t0,length(TO),sep="/"))
};end0=Sys.time()

end0-start0


ACC1[,c(4,5,7)]=ACC1[,c(4,5,7)]/100
ACC2[,c(4,5,7)]=ACC2[,c(4,5,7)]/100
ACC3[,c(4,5,7)]=ACC3[,c(4,5,7)]/100
ACC4[,c(4,5,7)]=ACC4[,c(4,5,7)]/100
ACC5[,c(4,5,7)]=ACC5[,c(4,5,7)]/100
ACC6[,c(4,5,7)]=ACC6[,c(4,5,7)]/100
ACC7[,c(4,5,7)]=ACC7[,c(4,5,7)]/100
ACC8[,c(4,5,7)]=ACC8[,c(4,5,7)]/100
ACC9[,c(4,5,7)]=ACC9[,c(4,5,7)]/100
ACC10[,c(4,5,7)]=ACC10[,c(4,5,7)]/100


ACC=list();for (i in 1:10){
  tmp=paste0("ACC",i)
  ACC[[i]]=round(eval(parse(text=tmp)),4)
}


save(ACC,FCST,Model1,Model2,Model3,Model4,Model5,Model6,Model7,
     Model8,Model9,Model10,testData,file="./output/data/arima_static.RData")




#Generate Forecasts of test data
print(load("./output/data/arima_static.RData"))

which.min(ACC[[8]][,"RMSE"])
Model8[[12]]
MODEL.used=Model8[[12]]$models[[length(Model8[[12]]$models)]]
fcst <- Arima(model=MODEL.used,testData)$fitted
acc=accuracy(fcst,x=as.ts(testData))
acc=t(as.matrix(acc))
acc[,c(4,5,7)]=acc[,c(4,5,7)]/100
acc=round(acc,4)
acc

ACCX=NULL
for (k in 1:13) { 
MODEL.used=Model8[[k]]$models[[length(Model8[[k]]$models)]]
fcst <- Arima(model=MODEL.used,testData)$fitted
accx=as.matrix(accuracy(fcst,x=as.ts(testData)))
accx[,c(4,5,7)]=accx[,c(4,5,7)]/100
accx=round(accx,4)
ACCX=rbind(ACCX,accx)
}
which.min(ACCX[,"RMSE"])


kfoldFCSTs=NULL
for (k in 1:13) {
  MODEL.used=Model8[[k]]$models[[length(Model8[[k]]$models)]]
  fcst <- Arima(model=MODEL.used,testData)$fitted
  kfoldFCSTs=cbind(kfoldFCSTs,fcst)
}
colnames(kfoldFCSTs)=paste0("Model",1:13)
fcstAvg=as.matrix(apply(kfoldFCSTs,1,mean))
rownames(fcstAvg)=as.character(time(testData))

accID=accuracy(as.ts(as.timeSeries(fcstAvg)),as.ts(testData))
accID[,"RMSE"]

which(sort(c(ACCX[,"RMSE"],accID[,"RMSE"]))==accID[,"RMSE"])

testData.timeID=time(testData)
testData=as.ts(testData)

kfoldAvg_all=NULL
for (m in 1:10) {

kfoldFCSTs=NULL
for (k in 1:13) { #m=8

  MODEL.used=eval(parse(text=paste0("Model",m)))[[k]]

    if (m==1) {
    
      fcst <- Arima(model=MODEL.used,testData)$fitted
    
    } else if (m==2) {
      fcst <- Arima(testData,model=MODEL.used,xreg=seasonaldummy(testData))$fitted
      
    } else if (m==3) {
    
      fcst <- Arima(model=MODEL.used, testData, xreg=fourier(testData, K=4))$fitted
      
    } else if (m==4) {
    
      fcst <- Arima(model=MODEL.used, testData,xreg=seq(testData))$fitted
      
    } else if (m==5) {
    
      fcst <- Arima(model=MODEL.used, testData,xreg=cbind(seasonaldummy(testData)))$fitted
      
    } else if (m==6) {
    
      fcst <- Arima(model=MODEL.used, testData,xreg=cbind(seasonaldummy(testData),seq(testData)))$fitted
      
    } else if (m==7) {
        MODEL.ets=MODEL.used$models[[length(MODEL.used$models)]]
      fcst <- baggedETS(model=MODEL.ets,testData, use.initial.values=TRUE)$fitted
    
    } else if (m==8) {
      MODEL.autoARIMA=MODEL.used$models[[length(MODEL.used$models)]]
      fcst <- Arima(model=MODEL.autoARIMA,testData, use.initial.values=TRUE)$fitted
      
    } else if (m==9) {
    
      fcst <- bats(model=MODEL.used,testData)$fitted
      
    } else if (m==10) {
      
      fcst <- arfima(model=MODEL.used,testData)$fitted
      
    }
  

  kfoldFCSTs=cbind(kfoldFCSTs,fcst)
} 
colnames(kfoldFCSTs)=paste0("Model",1:13)
kfoldAvg=as.matrix(apply(kfoldFCSTs,1,mean))
rownames(kfoldAvg)=as.character(time(testData))
colnames(kfoldAvg)=rownames(ACC[[m]])[1]

kfoldAvg_all=cbind(kfoldAvg_all,kfoldAvg)

} # End of m loop

rownames(kfoldAvg_all)=as.character(testData.timeID)

modelsAvg=as.matrix(apply(kfoldAvg_all,1,mean))
rownames(modelsAvg)=as.character(testData.timeID)
modelsAvg=as.timeSeries(modelsAvg)
colnames(modelsAvg)="modelsAverage"

ACC_modelsAvg=accuracy(as.ts(modelsAvg),testData)
