library(timeSeries)
print(load("./data/us_cpi.RData"))
y=na.omit(diff(log(us_cpi),12))*100
p=12
newData = embed(y,p+1)
dat = as.timeSeries(ts(newData,end=end(as.ts(y)),freq=frequency(y)))
data_training=window(dat,start=start(dat),end="2019-12-31")
data_testing=window(dat,start="2020-01-01",end="2021-09-03")
colnames(data_training)=colnames(data_testing)=c("dep",paste0("ar",1:(ncol(dat)-1)))

eq = as.formula(paste("dep ~", paste(colnames(data_training)[-1], collapse = "+")))

library(caret)

#Regression (decision) tree
out.tree=train(eq,data=data_training,method="rpart",metric = ifelse(is.factor(data_training[,"dep"]), "Accuracy", "MAE"))
tree.fcst1=as.timeSeries(predict(out.tree,data_testing))
forecast::accuracy(as.ts(tree.fcst1),x=as.ts(data_testing[,"dep"]))


#Random Forest
out.rf=train(eq,data=data_training,method="rf")
rf.fcst1=as.timeSeries(predict(out.rf,data_testing))
forecast::accuracy(as.ts(rf.fcst1),x=as.ts(data_testing[,"dep"]))

#K-Nearest Neighbors
out.knn=train(eq,data=data_training,method="kknn")
knn.fcst1=as.timeSeries(predict(out.knn,data_testing))
rownames(knn.fcst1)=time(data_testing)
knn.fcst1=as.timeSeries(knn.fcst1)
forecast::accuracy(as.ts(knn.fcst1),x=as.ts(data_testing[,"dep"]))


#Neural network
out.nnet=train(eq,data=data_training,method="nnet",
               tuneLength = 5,
               trace = FALSE,
               maxit = 100)
nnet.fcst1=as.timeSeries(predict(out.nnet,data_testing))
forecast::accuracy(as.ts(nnet.fcst1),x=as.ts(data_testing[,"dep"]))


#SVM

out.svm=train(eq, data=data_training, method="svmRadial",
                  tuneLength = 14,
                  trace = FALSE,
                  maxit = 100)
svm.fcst1=as.timeSeries(predict(out.svm,data_testing))
rownames(svm.fcst1)=time(data_testing)
forecast::accuracy(as.ts(svm.fcst1), x=as.ts(data_testing[,"dep"]))



#GBMs

ames = AmesHousing::make_ames()
set.seed(123)
split = rsample::initial_split(ames, prop = 0.7, 
                       strata = "Sale_Price")
ames_train  = rsample::training(split)
ames_test  = rsample::testing(split)

library(gbm)
ames_gbm1 = gbm(
  formula = Sale_Price ~ .,
  data = ames_train,
  distribution = "gaussian",  # SSE loss function
  n.trees = 5000,
  shrinkage = 0.1,
  interaction.depth = 3,
  n.minobsinnode = 10,
  cv.folds = 10
)

# find index for number trees with minimum CV error
best = which.min(ames_gbm1$cv.error)
names(ames_gbm1)

# get MSE and compute RMSE
sqrt(ames_gbm1$cv.error[best])
predict.err=predict(ames_gbm1,ames_test)-ames_test[,"Sale_Price"]
sqrt((sum(predict.err^2))/(nrow(ames_test)-1))


#Using train()
out1.GBMs=train(eq, data=data_training, method="gbm",
               trControl = trainControl(method = "cv"))
GBM.fcst1=as.matrix(predict(out1.GBMs,data_testing))
rownames(GBM.fcst1)=as.character(time(data_testing))
forecast::accuracy(as.ts(timeSeries(GBM.fcst1)), x=as.ts(data_testing[,"dep"]))





gbmGrid <- expand.grid(interaction.depth = 3,
                       n.trees = 5000,
                       shrinkage = 0.1,
                       n.minobsinnode=10)
set.seed(100)
out2.GBMs=train(eq, data=data_training, method="gbm",
               trControl = trainControl(method = "cv"),tuneGrid = gbmGrid)
out2.GBMs$results
GBM.fcst2=as.matrix(predict(out2.GBMs,data_testing))
rownames(GBM.fcst2)=as.character(time(data_testing))
forecast::accuracy(as.ts(timeSeries(GBM.fcst2)), x=as.ts(data_testing[,"dep"]))


gbmGrid <- expand.grid(interaction.depth = seq(1, 7, by = 2),
                       n.trees = seq(100, 1000, by = 50),
                       shrinkage = c(0.01, 0.1),
                       n.minobsinnode=10)


#lasso, ridge, & enet

#Using train()
lassoGrid =expand.grid(fraction = seq(.05, 1, length = 10))
out1.LASSO=train(eq, data=data_training,
                 method="lasso",
                 trControl = trainControl(method = "cv"))
LASSO.fcst1=as.matrix(predict(out1.LASSO,data_testing))
rownames(LASSO.fcst1)=as.character(time(data_testing))
forecast::accuracy(as.ts(timeSeries(LASSO.fcst1)), x=as.ts(data_testing[,"dep"]))


ridgeGrid = expand.grid(lambda = seq(0, .2, length = 50))
out1.ridge=train(eq, data=data_training,
                 method="ridge",
                 tuneLength=20,
                 trControl = trainControl(method = "cv"))
RIDGE.fcst1=as.matrix(predict(out1.ridge,data_testing))
rownames(RIDGE.fcst1)=as.character(time(data_testing))
forecast::accuracy(as.ts(timeSeries(RIDGE.fcst1)), x=as.ts(data_testing[,"dep"]))



enetGrid = expand.grid(lambda = c(0, 0.01, .1),
                        fraction = seq(.05, 1, length = 20))
set.seed(100)
out1.ENET=train(eq, data=data_training, method="enet",
                trControl = trainControl(method = "cv"))
ENET.fcst1=as.matrix(predict(out1.ENET,data_testing))
rownames(ENET.fcst1)=as.character(time(data_testing))
forecast::accuracy(as.ts(timeSeries(ENET.fcst1)), x=as.ts(data_testing[,"dep"]))


#H2O autoML
library(iForecast)
# Choosing the dates of training and testing data
train.end="2019-12-01"
test.start="2020-01-01"

autoML=ttsAutoML(y=y, x=NULL, train.end,arOrder=c(1:12), xregOrder=c(0), maxSecs =30)

testData=window(autoML$data, start=test.start,end=end(autoML$data))
autoML.fcst=iForecast(Model=autoML,newdata=testData,type="staticfit")

tail(cbind(testData[,1],autoML.fcst),10)

forecast::accuracy(as.ts(testData[,1]), as.ts(autoML.fcst))




#Dynamic/Recursive Forecast
train.end="2019-12-01"
test.start="2020-01-01"

type=c("none","trend","season","both")[1]
models=c("rf","rpart","svm","knn","nnet","gbm","lasso","ridge","enet")[3]
out.caret = ttsCaret(y=y, x=NULL, arOrder=c(1:6), method=models,
                 tuneLength =10, train.end, type=type)
head(out.caret$dataused)

testData=window(out.caret$data,start=test.start,end=end(out.caret$data))


P1=iForecast(Model=out.caret,newdata=testData,type="staticfit") # static forecasts generated
P2=iForecast(Model=out.caret,newdata=testData,type="recursive") # dynamic forecasts generated

cbind(testData[,1],P1,P2)


trainData=window(out.caret$dataused,start=start(out.caret$dataused),end=train.end)

P0=iForecast(Model=out.caret, newdata=trainData, type="staticfit")

sample0=cbind(actual=trainData[,1],pred=P0)
sample1=cbind(actual=testData[,1],pred=P1)
newDataX=rbind(sample0, sample1)
rownames(newDataX)=c(rownames(P0),rownames(testData))
newDataX=as.timeSeries(newDataX)
t0=which(as.character(time(newDataX))==train.end)
dev.new()
plot(newDataX[,1],ylim=range(newDataX),main="",ylab="",xlab="Static Forecasts",lty = 1)
lines(newDataX[,2],ylim=range(newDataX), col="red", pch=1,type="o",lty = 2)
legend("topleft", legend=c("Actual Data","Static Forecast"),text.col=c("black","red"),pch=c(NA,1),lty=c(1,2))
abline(h=0,v=as.POSIXct(test.start),col="blue");grid()
text(as.POSIXct(rownames(newDataX)[t0-20]), max(dep)*0.9, test.start,col="blue")




autoML=ttsAutoML(y=y , x=NULL, train.end, arOrder=c(1:6), xregOrder=c(1,3), maxSecs =300)
testData=window(autoML$data, start=test.start, end=end(autoML$data))
P1=iForecast(Model=autoML, newdata=testData, type="staticfit")
P2=iForecast(Model=autoML, newdata=testData, type="recursive")

tail(cbind(testData[,1],P1,P2),10)






data(package = "iForecast",macrodata)
head(macrodata)
dep=macrodata[,1,drop=FALSE] #unemployment rate of USA
ind=macrodata[,-1,drop=FALSE]
train.end="2017-12-01"
test.start="2018-01-01"

type=c("none","trend","season","both")[1]
models=c("rf","rpart","svm","knn","nnet","gbm","lasso","ridge","enet")[1]
out.caret = ttsCaret(y=dep, x=ind, arOrder=c(1:3), xregOrder=c(0:3),method=models,
                     tuneLength =10, train.end, type=type,
                  resampling=c("boot","cv","repeatedcv")[1],preProcess = "center")
head(out.caret$dataused)

testData=window(out.caret$data,start=test.start,end=end(out.caret$data))

P1=iForecast(Model=out.caret,newdata=testData,type="staticfit") # static forecasts generated
P2=iForecast(Model=out.caret,newdata=testData,type="recursive") # dynamic forecasts generated

tail(cbind(testData[,1],P1),10)
tail(cbind(testData[,1],P2),10)



#Recursively extending to real future
train.end="2019-12-01"
test.start="2020-01-01"

type=c("none","trend","season","both")[1]
models=c("rf","rpart","svm","knn","nnet","gbm","lasso","ridge","enet")[3]
out.caret = ttsCaret(y=y, x=NULL, arOrder=c(1:6), method=models,
                     tuneLength =10, train.end, type=type,
                  resampling=c("boot","cv","repeatedcv")[1],preProcess = "center")
head(out.caret$dataused)

testData=window(out.caret$data,start=test.start,end=end(out.caret$data))
end(testData)+c(24*60*60,24*60*60*2)

P1=iForecast(Model=out.caret,newdata=testData,type="staticfit") # static forecasts generated
P2=iForecast(Model=out.caret,newdata=testData,type="recursive") # dynamic forecasts generated

tail(cbind(testData[,1],P1),10)
tail(cbind(testData[,1],P2),10)


