#setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
##===== Codes below generate  multistep forecasts
library(iForecast)
library(forecast)
library(timeSeries)
Sys.setlocale(category = "LC_ALL", locale = "English_United States.1252")

print(load("./data/indexReturns.RData"))
#indexReturns=as.zoo(indexReturns)
#economicForces=as.zoo(economicForces)
Y=indexReturns[,2,drop=FALSE];tail(Y);head(Y)
datainput=cbind(Y,economicForces)
head(datainput)

train.end="2021-01-01"
test.start="2021-02-01"
tainingData=window(datainput,start=start(datainput),end=train.end)
testData=window(datainput,start=test.start,end=end(datainput))
nrow(tainingData)
nrow(testData)

#####==== ML Case 1. =====#####
type=c("none","trend","season","both")[1]

timeframe=rollingWindows(tainingData,estimation="240m",by="6m")
FROM=timeframe$from
TO=timeframe$to



#dep=Y
models=c("svm","rf","rpart","gbm","kknn","bridge","relaxo","blasso","enet")

start0=Sys.time()
model.svm=model.rf=model.rpart=model.gbm=model.kknn=model.bridge=
  model.relaxo=model.blasso=model.enet=list()
ACC_static=ACC_recursive=FCST_static=FCST_recursive=list()
for (t0 in 1:(length(TO)-1)) { #  t0=1

ACC0_static=ACC0_recursive=predicted_static=predicted_recursive=NULL  
for (i in 1:length(models)) { #i=1
  
output = ttsCaret(y=tainingData[,1], x=tainingData[,-1], method=models[i],
                  arOrder=1, xregOrder=c(1), tuneLength =10, 
                   train.end=as.character(TO[t0]), type=type,
                  resampling=c("boot","cv","repeatedcv")[1],preProcess = "center")

if(t0==(length(TO)-1)) {
  validationData=window(output$dataused,start=TO[t0],end=end(tainingData))[-1,]
} else {
  validationData=window(output$dataused,start=TO[t0],end=TO[t0+1])[-1,]

}

if (models[i]=="svm") {model.svm[[t0]]=output
    } else if (models[i]=="rf") {model.rf[[t0]]=output
    } else if (models[i]=="rpart") {model.rpart[[t0]]=output
    } else if (models[i]=="gbm") {model.gbm[[t0]]=output
    } else if (models[i]=="kknn") {model.kknn[[t0]]=output
    } else if (models[i]=="bridge") {model.bridge[[t0]]=output
    } else if (models[i]=="relaxo") {model.relaxo[[t0]]=output
    } else if (models[i]=="blasso") {model.blasso[[t0]]=output
    } else if (models[i]=="enet") {model.enet[[t0]]=output} 

P1=iForecast(Model=output,newdata=validationData,type="staticfit") # static forecasts generated
acc_static=forecast::accuracy(as.ts(validationData[,1]),as.ts(P1))
rownames(acc_static)=paste(models[i],"static",t0,sep="_")
ACC0_static=rbind(ACC0_static,acc_static)
predicted_static=cbind(predicted_static,P1)

P2=iForecast(Model=output,newdata=validationData,type="recursive") # dynamic forecasts generated
acc_recursive=forecast::accuracy(as.ts(validationData[,1]),as.ts(P2))
rownames(acc_recursive)=paste(models[i],"recursive",t0,sep="_")
ACC0_recursive=rbind(ACC0_recursive,acc_recursive)
predicted_recursive=cbind(predicted_recursive,P2)

} # End of i loop for methods

fcst_static=cbind(validationData[,1],predicted_static)
colnames(fcst_static)=c("ya",models)
FCST_static[[t0]]=fcst_static
ACC0_static[,c(4,5,7)]=ACC0_static[,c(4,5,7)]/100
rownames(ACC0_static)=models
ACC_static[[t0]]=ACC0_static

fcst_recursive=cbind(validationData[,1],predicted_recursive)
colnames(fcst_recursive)=c("ya",models)
FCST_recursive[[t0]]=fcst_recursive
ACC0_recursive[,c(4,5,7)]=ACC0_recursive[,c(4,5,7)]/100
rownames(ACC0_recursive)=models
ACC_recursive[[t0]]=ACC0_recursive

print(paste(t0,length(TO),sep="/"))
}; end0=Sys.time()# End of t0 loop for time

end0-start0 

#save(ACC_static,FCST_static,model.svm, model.rf, model.rpart,model.gbm,model.kknn,model.bridge,
#     model.relaxo,model.blasso,model.enet,testData,file=paste0("./output/data/svm_static.RData"))

save(model.svm, model.rf, model.rpart,model.gbm,model.kknn,model.bridge,
     model.relaxo,model.blasso,model.enet,testData,file=paste0("./output/data/svm_ar1_Models.RData"))
save(ACC_static,FCST_static,testData,file=paste0("./output/data/svm_ar1_static.RData"))
save(ACC_recursive,FCST_recursive,testData,file=paste0("./output/data/svm_ar1_recursive.RData"))






library(iForecast)
library(forecast)
library(timeSeries)
Sys.setlocale(category = "LC_ALL", locale = "English_United States.1252")
#== ML Case 2. autoML of H2O.ai
print(load("./data/myData.RData"))
Y=indexReturns[,1,drop=FALSE];tail(Y);head(Y)
datainput=cbind(Y,economicForces)
head(datainput)

train.end="2021-01-01"
test.start="2021-02-01"
tainingData=window(datainput,start=start(Y),end=train.end)
testData=window(datainput,start=test.start,end=end(Y))
nrow(tainingData)
nrow(testData)

timeframe=rollingWindows(tainingData,training="300m",testing="6m")
FROM=timeframe$from
TO=timeframe$to


model.autoML=list()
FCST_static=FCST_recursive=list()
ACC_static=ACC_recursive=NULL
start0=Sys.time();for (t0 in 1:(length(TO)-1)) { #  t0=1
  
  output=ttsAutoML(y=tainingData[,1], x=tainingData[,-1], arOrder=1, xregOrder=1,
                   train.end=as.character(TO[t0]),maxSecs =30)
    
  if(t0==(length(TO)-1)) {
    validationData=window(output$dataused,start=TO[t0],end=end(tainingData))[-1,]
  } else {
    validationData=window(output$dataused,start=TO[t0],end=TO[t0+1])[-1,]
    
  }

P1=iForecast(Model=output,newdata=validationData,type="staticfit")
acc_static=forecast::accuracy(as.ts(validationData[,1]),as.ts(P1))
rownames(acc_static)=paste("autoML","static",t0,sep="_")
ACC_static=rbind(ACC_static,acc_static)
fcst_static=cbind(testData[,1],P1)
colnames(fcst_static)=c("ya","static")
FCST_static[[t0]]=fcst_static

P2=iForecast(Model=output,newdata=validationData,type="recursive")
acc_recursive=forecast::accuracy(as.ts(validationData[,1]),as.ts(P2))
rownames(acc_recursive)=paste("autoML","recursive",t0,sep="_")
ACC_recursive=rbind(ACC_recursive,acc_recursive)
fcst_recursive=cbind(testData[,1],P2)
colnames(fcst_recursive)=c("ya","recursive")
FCST_recursive[[t0]]=fcst_recursive

model.autoML[[t0]]=output

print(paste(t0,(length(TO)-1),sep="/"))
};end0=Sys.time()

end0-start0

ACC_static[,c(4,5,7)]=ACC_static[,c(4,5,7)]/100
ACC_recursive[,c(4,5,7)]=ACC_recursive[,c(4,5,7)]/100




savedfilenames0=paste0("./output/data/autoML_ar1_model.RData")
savedfilenames1=paste0("./output/data/autoML_ar1_static.RData")
savedfilenames2=paste0("./output/data/autoML_ar1_recursive.RData")
save(model.autoML,testData,file=savedfilenames0)
save(ACC_static,FCST_static,testData,file=savedfilenames1)
save(ACC_recursive,FCST_recursive,testData,file=savedfilenames2)


savedfilenames0=paste0("./output/data/autoML_static_models.RData")
savedfilenames1=paste0("./output/data/autoML_static.RData")
save(model.autoML,testData,file=savedfilenames0)
save(ACC_static,FCST_static,testData,file=savedfilenames1)














#== ML Case 3. RNN-LSTM: binary dependent variable is NOT allowed so far
source("./src/rollingWindows.R")
library(iForecast)
library(forecast)
library(timeSeries)
#Sys.setlocale(category = "LC_ALL", locale = "English_United States.1252")

print(load("./data/myData.RData"))
indexReturns=as.zoo(indexReturns)
economicForces=as.zoo(economicForces)
Y=indexReturns[,1,drop=FALSE];tail(Y);head(Y)
datainput=cbind(Y,economicForces)
head(datainput)

train.end="2021-01-01"
test.start="2021-02-01"
tainingData=window(datainput,start=start(Y),end=train.end)
testData=window(datainput,start=test.start,end=end(Y))
nrow(tainingData)
nrow(testData)

type=c("none","trend","season","both")[1]

timeframe=rollingWindows(tainingData,period="240m",by="6m")
FROM=timeframe$from
TO=timeframe$to


source("./src/ttsLSTM.R")

FCST_static=FCST_recursive=list()
fcst_static=fcst_recursive=ACC_static=ACC_recursive=NULL
start0=Sys.time();for (t0 in 1:length(TO)) { #   t0=11  (11,15,24,34,37,50,53,63,73,76)

  output=ttsLSTM(y=tainingData[,1], x=tainingData[,-1], arOrder=0, xregOrder=1,
               train.end=as.character(TO[t0]),memoryLoops=180,type=type)

    if(t0==length(TO)) {
      validationData=window(output$dataused,start=TO[t0],end=end(tainingData))[-1,]
    } else {validationData=window(output$dataused,start=TO[t0],end=TO[t0+1])[-1,]}
 
  P1= iForecast(Model=output,newdata=validationData,type="staticfit")
  P2= iForecast(Model=output,newdata=validationData,type="recursive")
  
  acc_static=forecast::accuracy(as.ts(validationData[,1]),as.ts(P1))
  acc_recursive=forecast::accuracy(as.ts(validationData[,1]),as.ts(P2))
  rownames(acc_static)=paste("LSTM","static",t0,sep="_")
  rownames(acc_recursive)=paste("LSTM","recursive",t0,sep="_")
  ACC_static=rbind(ACC_static,acc_static)
  ACC_recursive=rbind(ACC_recursive,acc_recursive)
  
  fcst_static=cbind(validationData[,1],P1)
  fcst_recursive=cbind(validationData[,1],P2)
  colnames(fcst_static)=c("ya","static")
  colnames(fcst_recursive)=c("ya","recursive")
  
  FCST_static[[t0]]=fcst_static
  FCST_recursive[[t0]]=fcst_recursive
  
  
  print(paste(t0,length(TO),sep="/"))
};end0=Sys.time()

end0-start0

ACC_static[,c(4,5,7)]=ACC_static[,c(4,5,7)]/100
ACC_recursive[,c(4,5,7)]=ACC_recursive[,c(4,5,7)]/100

savedfilenames1=paste0("./output/data/LSTM_ar1_static.RData")
savedfilenames2=paste0("./output/data/LSTM_ar1_recursive.RData")
save(ACC_static,FCST_static,testData,file=savedfilenames1)
save(ACC_recursive,FCST_recursive,testData,file=savedfilenames2)

