library(iForecast)
## Data type 1. 
data(macrodata) # Continuous dependent variable,monthly
class(macrodata) #zoo object
dep=macrodata[,1,drop=FALSE] #unemployment rate of USA
ind=macrodata[,-1,drop=FALSE]
# Choosing the dates of training and testing data
train.end=as.character(rownames(dep))[as.integer(nrow(dep)*0.8)]
test.start=as.character(rownames(dep))[as.integer(nrow(dep)*0.8)+1]

dev.new();plot(timeSeries::as.timeSeries(dep), col="steelblue",main="",ylab="Value, %");grid()
abline(v=as.POSIXct(test.start),col="red")
text(as.POSIXct(test.start), max(dep)*0.95, test.start,col="blue")

## Data type 2
data(bc) # Binary dependent variable,monthly
class(bc) #zoo object
dep=bc[,1,drop=FALSE] #Taiwan's business cycle phases
ind=bc[,-1]
# Choosing the dates of training and testing data
train.end=as.character(rownames(dep))[as.integer(nrow(dep)*0.8)]
test.start=as.character(rownames(dep))[as.integer(nrow(dep)*0.8)+1]

## Data type 3
#data(ES_15m) # high-freq, 15-minute time series, 
              # realized absolute variance (RAV) of e-mini S&P 500
#dep=ES_15m[1:1000,1]
#ind=NULL
# Choosing the dates of training and testing data
#train.end=as.character(time(dep))[as.integer(nrow(dep)*0.9)]
#test.start=as.character(time(dep))[as.integer(nrow(dep)*0.9)+1]


#== ML Case 1. Machine Learning by library(caret);library(zoo)
type=c("none","trend","season","both")[1]
models=c("rf","rpart","svm","kknn","glm","nnet")[4]
Caret = ttsCaret(y=dep, x=ind, arOrder=c(1), xregOrder=c(1,2), method=models,
                   tuneLength =10, train.end, type=type)

testData1=window(Caret$data,start=test.start,end=end(Caret$data))

head(Caret$dataused)
P1=iForecast(Model=Caret,newdata=testData1,type="staticfit") # static forecasts generated
P2=iForecast(Model=Caret,newdata=testData1,type="recursive") # dynamic forecasts generated

tail(cbind(testData1[,1],P1),10)
tail(cbind(testData1[,1],P2),10)

#== ML Case 2. autoML of H2O.ai
autoML=ttsAutoML(y=dep, x=ind, train.end,arOrder=c(1), xregOrder=c(1,3), maxSecs =30)

testData2=window(autoML$data,start=test.start,end=end(autoML$data))
P1=iForecast(Model=autoML,newdata=testData2,type="staticfit")
P2=iForecast(Model=autoML,newdata=testData2,type="recursive")

tail(cbind(testData2[,1],P1),10)
tail(cbind(testData2[,1],P2),10)


#== ML Case 3. RNN-LSTM: binary dependent variable is NOT allowed so far
LSTM=ttsLSTM(y=dep, x=ind, train.end,arOrder=c(1), xregOrder=c(0,2,4), memoryLoops=25,type=type)

testData3=window(LSTM$data,start=test.start,end=end(LSTM$data))
P1=iForecast(Model=LSTM,newdata=testData3,type="staticfit")
P2=iForecast(Model=LSTM,newdata=testData3,type="recursive")

tail(cbind(testData3[,1],P1),10)
tail(cbind(testData3[,1],P2),10)

